@@ -470,8 +470,52 @@ def test_find_missing_imports_class_name_2():
     assert expected == result
 
 
+@pytest.mark.xfail
+def test_find_missing_import_xfail_after_pr_152():
+    code = dedent(
+        """
+        class MyClass(object):
+            Outer = MyClass
+    """
+    )
+    result = find_missing_imports(code, [{}])
+    result = _dilist2strlist(result)
+    expected = ["MyClass"]
+    assert result == expected
+
+
+def test_method_reference_current_class():
+    """
+    A method can reference the current class
+
+    But only if this is a toplevel class, nesting won't work in Python.
+    """
+    code = dedent(
+        """
+       class Decimal:
+           def foo(self):
+               Decimal.x = 1
+               Float.y=1
+
+           class Real:
+
+               def foo():
+                   Real.r = 1
+   """
+    )
+    missing, unused = scan_for_import_issues(code, [{}])
+    # result = _dilist2strlist(result)
+    assert missing == [
+        (5, DottedIdentifier("Float.y")),
+        (10, DottedIdentifier("Real.r")),
+    ]
+    assert unused == []
+
+
+
 def test_find_missing_imports_class_name_1():
-    code = dedent("""
+    code = dedent(
+        """
         class Corinne(object):
             pass
         class Bobtail(object):
@@ -485,6 +529,7 @@ def test_find_missing_imports_class_name_1():
     assert expected == result
 
 
+
 def test_find_missing_imports_class_members_1():
     code = dedent("""
         class Kenosha(object):
@@ -1237,6 +1282,52 @@ def test_scan_for_import_issues_type_comment_1():
     assert unused == []
     assert missing == []
 
+
+# Only Python 3.8 includes type comments in the ast, so we only support this
+# there (see issue #31, 171, 174).
+@pytest.mark.skipif(
+    sys.version_info < (3, 8),
+    reason="Python 3.8+-only support.")
+def test_scan_for_import_issues_type_comment_2():
+    code = dedent("""
+    from typing import Sequence
+    def foo(strings):
+        # type: (Sequence[str]) -> None
+        pass
+    """)
+    missing, unused = scan_for_import_issues(code)
+    assert unused == []
+    assert missing == []
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 8),
+    reason="Python 3.8+-only support.")
+def test_scan_for_import_issues_type_comment_3():
+    code = dedent("""
+    def foo(strings):
+        # type: (Sequence[str]) -> None
+        pass
+    """)
+    missing, unused = scan_for_import_issues(code)
+    assert unused == []
+    assert missing == [(1, DottedIdentifier('Sequence'))]
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 8),
+    reason="Python 3.8+-only support.")
+def test_scan_for_import_issues_type_comment_4():
+    code = dedent("""
+    from typing import Sequence, Tuple
+    def foo(strings):
+        # type: (Sequence[str]) -> None
+        pass
+    """)
+    missing, unused = scan_for_import_issues(code)
+    assert unused == [(2, Import('from typing import Tuple'))]
+    assert missing == []
+
 # Python 3.8 uses the correct line number for multiline strings (the first
 # line), making _annotate_ast_startpos irrelevant. Otherwise, the logic for
 # getting this right is too hard. See issue #12.
